@@ -26,6 +26,8 @@ from lxml import etree as ETREE
 from netaddr import IPAddress  # needed for traceroute, to check IP version
 from netaddr.core import AddrFormatError
 
+from netmiko import __version__ as netmiko_version
+
 from pyIOSXR import IOSXR
 from pyIOSXR.exceptions import ConnectError
 from pyIOSXR.exceptions import TimeoutError
@@ -57,13 +59,41 @@ class IOSXRDriver(NetworkDriver):
         if optional_args is None:
             optional_args = {}
         self.port = optional_args.get('port', 22)
-        self.lock_on_connect = optional_args.get('config_lock', True)
+        self.lock_on_connect = optional_args.get('config_lock', False)
+        # Netmiko possible arguments
+        netmiko_argument_map = {
+            'keepalive': 30,
+            'verbose': False,
+            'global_delay_factor': 1,
+            'use_keys': False,
+            'key_file': None,
+            'ssh_strict': False,
+            'system_host_keys': False,
+            'alt_host_keys': False,
+            'alt_key_file': '',
+            'ssh_config_file': None
+        }
+        fields = netmiko_version.split('.')
+        fields = [int(x) for x in fields]
+        maj_ver, min_ver, bug_fix = fields
+        if maj_ver >= 2:
+            netmiko_argument_map['allow_agent'] = False
+        elif maj_ver == 1 and min_ver >= 1:
+            netmiko_argument_map['allow_agent'] = False
+        # Build dict of any optional Netmiko args
+        self.netmiko_optional_args = {}
+        for k, v in netmiko_argument_map.items():
+            try:
+                self.netmiko_optional_args[k] = optional_args[k]
+            except KeyError:
+                pass
         self.device = IOSXR(hostname,
                             username,
                             password,
                             timeout=timeout,
                             port=self.port,
-                            lock=self.lock_on_connect)
+                            lock=self.lock_on_connect,
+                            **self.netmiko_optional_args)
 
     def open(self):
         try:
@@ -75,8 +105,10 @@ class IOSXRDriver(NetworkDriver):
         self.device.close()
 
     def is_alive(self):
+        """Returns a flag with the state of the connection."""
+        # Simply returns the flag from Netmiko
         return {
-            'is_alive': self.device.device.remote_conn.transport.is_active()
+            'is_alive': self.device.device.is_alive()
         }
 
     def load_replace_candidate(self, filename=None, config=None):
@@ -151,7 +183,10 @@ class IOSXRDriver(NetworkDriver):
         system_time_xpath = './/SystemTime/Uptime'
         platform_attr_xpath = './/RackTable/Rack/Attributes/BasicInfo'
         system_time_tree = facts_rpc_reply.xpath(system_time_xpath)[0]
-        platform_attr_tree = facts_rpc_reply.xpath(platform_attr_xpath)[0]
+        try:
+            platform_attr_tree = facts_rpc_reply.xpath(platform_attr_xpath)[0]
+        except IndexError:
+            platform_attr_tree = facts_rpc_reply.xpath(platform_attr_xpath)
 
         hostname = napalm_base.helpers.convert(
             text_type, napalm_base.helpers.find_txt(system_time_tree, 'Hostname'))
@@ -680,9 +715,9 @@ class IOSXRDriver(NetworkDriver):
             inet = False
             inet6 = False
             preifx_type = 'inet'
-            if 'IPV4' in af_table:
+            if 'ipv4' in af_table.lower():
                 inet = True
-            if 'IPv6' in af_table:
+            if 'ipv6' in af_table.lower():
                 inet6 = True
                 preifx_type = 'inet6'
             if inet or inet6:
@@ -775,8 +810,8 @@ class IOSXRDriver(NetworkDriver):
                     bgp_group, 'NeighborGroupAFTable/NeighborGroupAF/RoutePolicyIn')
             export_policy = napalm_base.helpers.find_txt(
                     bgp_group, 'NeighborGroupAFTable/NeighborGroupAF/RoutePolicyOut')
-            multipath = eval((napalm_base.helpers.find_txt(
-                bgp_group, 'NeighborGroupAFTable/NeighborGroupAF/Multipath') or 'false').title())
+            multipath = napalm_base.helpers.find_txt(
+                bgp_group, 'NeighborGroupAFTable/NeighborGroupAF/Multipath') == 'true'
             peer_as = napalm_base.helpers.convert(
                 int, napalm_base.helpers.find_txt(bgp_group, 'RemoteAS/AS_YY'), 0)
             local_as = napalm_base.helpers.convert(
@@ -820,6 +855,22 @@ class IOSXRDriver(NetworkDriver):
             }
             if group and group == group_name:
                 break
+        if '' in bgp_group_neighbors.keys():
+            bgp_config['_'] = {
+                'apply_groups': [],
+                'description': '',
+                'local_as': 0,
+                'type': '',
+                'import_policy': '',
+                'export_policy': '',
+                'local_address': '',
+                'multipath': False,
+                'multihop_ttl': 0,
+                'remote_as': 0,
+                'remove_private_as': False,
+                'prefix_limit': {},
+                'neighbors': bgp_group_neighbors.get('', {})
+            }
 
         return bgp_config
 
@@ -890,8 +941,8 @@ class IOSXRDriver(NetworkDriver):
                     napalm_base.helpers.find_txt(neighbor, 'Naming/NeighborAddress/IPV4Address') or
                     napalm_base.helpers.find_txt(neighbor, 'Naming/NeighborAddress/IPV6Address')
                 )
-                local_address_configured = eval(napalm_base.helpers.find_txt(
-                    neighbor, 'IsLocalAddressConfigured', 'false').title())
+                local_address_configured = napalm_base.helpers.find_txt(
+                    neighbor, 'IsLocalAddressConfigured') == 'true'
                 local_address = napalm_base.helpers.ip(
                     napalm_base.helpers.find_txt(neighbor, 'ConnectionLocalAddress/IPV4Address') or
                     napalm_base.helpers.find_txt(neighbor, 'ConnectionLocalAddress/IPV6Address')
@@ -904,12 +955,12 @@ class IOSXRDriver(NetworkDriver):
                 )
                 remote_port = napalm_base.helpers.convert(
                     int, napalm_base.helpers.find_txt(neighbor, 'ConnectionRemotePort'))
-                multihop = eval(napalm_base.helpers.find_txt(
-                    neighbor, 'IsExternalNeighborNotDirectlyConnected', 'false').title())
-                remove_private_as = eval(napalm_base.helpers.find_txt(
-                    neighbor, 'AFData/Entry/RemovePrivateASFromUpdates', 'false').title())
-                multipath = eval(napalm_base.helpers.find_txt(
-                    neighbor, 'AFData/Entry/SelectiveMultipathEligible', 'false').title())
+                multihop = napalm_base.helpers.find_txt(
+                    neighbor, 'IsExternalNeighborNotDirectlyConnected') == 'true'
+                remove_private_as = napalm_base.helpers.find_txt(
+                    neighbor, 'AFData/Entry/RemovePrivateASFromUpdates') == 'true'
+                multipath = napalm_base.helpers.find_txt(
+                    neighbor, 'AFData/Entry/SelectiveMultipathEligible') == 'true'
                 import_policy = napalm_base.helpers.find_txt(
                     neighbor, 'AFData/Entry/RoutePolicyIn')
                 export_policy = napalm_base.helpers.find_txt(
@@ -941,10 +992,10 @@ class IOSXRDriver(NetworkDriver):
                 advertised_prefix_count = napalm_base.helpers.convert(
                     int, napalm_base.helpers.find_txt(
                         neighbor, 'AFData/Entry/PrefixesAdvertised'), 0)
-                suppress_4byte_as = eval(napalm_base.helpers.find_txt(
-                    neighbor, 'Suppress4ByteAs', 'false').title())
-                local_as_prepend = not eval(napalm_base.helpers.find_txt(
-                    neighbor, 'LocalASNoPrepend', 'false').title())
+                suppress_4byte_as = napalm_base.helpers.find_txt(
+                    neighbor, 'Suppress4ByteAs') == 'true'
+                local_as_prepend = napalm_base.helpers.find_txt(
+                    neighbor, 'LocalASNoPrepend') != 'true'
                 holdtime = napalm_base.helpers.convert(
                     int, napalm_base.helpers.find_txt(neighbor, 'HoldTime'), 0) or vrf_holdtime
                 configured_holdtime = napalm_base.helpers.convert(
@@ -996,7 +1047,7 @@ class IOSXRDriver(NetworkDriver):
                     'advertised_prefix_count': advertised_prefix_count,
                     'flap_count': flap_count
                 })
-
+        bgp_neighbors_detail['global'] = bgp_neighbors_detail.pop('default')
         return bgp_neighbors_detail
 
     def get_arp_table(self):
@@ -1083,7 +1134,7 @@ class IOSXRDriver(NetworkDriver):
 
         xpath = './/NodeTable/Node/Associations/PeerSummaryInfo/Entry/PeerInfoCommon'
         for node in result_tree.xpath(xpath):
-            synchronized = eval(napalm_base.helpers.find_txt(node, 'IsSysPeer', 'false').title())
+            synchronized = napalm_base.helpers.find_txt(node, 'IsSysPeer') == 'true'
             address = napalm_base.helpers.find_txt(node, 'Address')
             if address == 'DLRSC node':
                 continue
@@ -1224,15 +1275,30 @@ class IOSXRDriver(NetworkDriver):
             prefix_tag = '<PrefixLength>{prefix_length}</PrefixLength>'.format(
                 prefix_length=dest_split[1])
 
-        route_info_rpc_command = '<Get><Operational><RIB><VRFTable><VRF><Naming><VRFName>default\
-        </VRFName></Naming><AFTable><AF><Naming><AFName>IPv4</AFName></Naming><SAFTable><SAF>\
-        <Naming><SAFName>Unicast</SAFName></Naming><IP_RIBRouteTable><IP_RIBRoute><Naming>\
-        <RouteTableName>default</RouteTableName></Naming><RouteTable><Route><Naming><Address>\
-        {network}</Address>{prefix}</Naming></Route></RouteTable></IP_RIBRoute></IP_RIBRouteTable>\
-        </SAF></SAFTable></AF></AFTable></VRF></VRFTable></RIB></Operational></Get>'.format(
-            network=network,
-            prefix=prefix_tag
-        )
+        ipv = 4
+        try:
+            ipv = IPAddress(network).version
+        except AddrFormatError:
+            raise TypeError('Wrong destination IP Address!')
+
+        if ipv == 6:
+            route_info_rpc_command = '<Get><Operational><IPV6_RIB><VRFTable><VRF><Naming><VRFName>\
+            default</VRFName></Naming><AFTable><AF><Naming><AFName>IPv6</AFName></Naming><SAFTable>\
+            <SAF><Naming><SAFName>Unicast</SAFName></Naming><IP_RIBRouteTable><IP_RIBRoute><Naming>\
+            <RouteTableName>default</RouteTableName></Naming><RouteTable><Route><Naming><Address>\
+            {network}</Address>{prefix}</Naming></Route></RouteTable></IP_RIBRoute>\
+            </IP_RIBRouteTable></SAF></SAFTable></AF></AFTable></VRF></VRFTable></IPV6_RIB>\
+            </Operational></Get>'.format(network=network, prefix=prefix_tag)
+        else:
+            route_info_rpc_command = '<Get><Operational><RIB><VRFTable><VRF><Naming><VRFName>default\
+            </VRFName></Naming><AFTable><AF><Naming><AFName>IPv4</AFName></Naming><SAFTable><SAF>\
+            <Naming><SAFName>Unicast</SAFName></Naming><IP_RIBRouteTable><IP_RIBRoute><Naming>\
+            <RouteTableName>default</RouteTableName></Naming><RouteTable><Route><Naming><Address>\
+            {network}</Address>{prefix}</Naming></Route></RouteTable></IP_RIBRoute></IP_RIBRouteTable>\
+            </SAF></SAFTable></AF></AFTable></VRF></VRFTable></RIB></Operational></Get>'.format(
+                network=network,
+                prefix=prefix_tag
+            )
 
         routes_tree = ETREE.fromstring(self.device.make_rpc_call(route_info_rpc_command))
 
@@ -1291,8 +1357,8 @@ class IOSXRDriver(NetworkDriver):
                 for bgp_path in bgp_route_tree.xpath('.//Path'):
                     single_route_details = route_details.copy()
                     if 'NotFound' not in bgp_path.keys():
-                        best_path = eval(napalm_base.helpers.find_txt(
-                            bgp_path, 'PathInformation/IsBestPath', 'false').title())
+                        best_path = napalm_base.helpers.find_txt(
+                            bgp_path, 'PathInformation/IsBestPath') == 'true'
                         local_preference = napalm_base.helpers.convert(
                             int,
                             napalm_base.helpers.find_txt(
@@ -1558,7 +1624,8 @@ class IOSXRDriver(NetworkDriver):
                    destination,
                    source=C.TRACEROUTE_SOURCE,
                    ttl=C.TRACEROUTE_TTL,
-                   timeout=C.TRACEROUTE_TIMEOUT):
+                   timeout=C.TRACEROUTE_TIMEOUT,
+                   vrf=C.TRACEROUTE_VRF):
 
         traceroute_result = {}
 
@@ -1571,20 +1638,22 @@ class IOSXRDriver(NetworkDriver):
         source_tag = ''
         ttl_tag = ''
         timeout_tag = ''
+        vrf_tag = ''
         if source:
             source_tag = '<Source>{source}</Source>'.format(source=source)
         if ttl:
             ttl_tag = '<MaxTTL>{maxttl}</MaxTTL>'.format(maxttl=ttl)
         if timeout:
             timeout_tag = '<Timeout>{timeout}</Timeout>'.format(timeout=timeout)
-        else:
-            timeout = 5  # seconds
+        if vrf:
+            vrf_tag = '<VRFName>{vrf}</VRFName>'.format(vrf=vrf)
 
         traceroute_rpc_command = '<Set><Action><TraceRoute><IPV{version}><Destination>{destination}\
-        </Destination>{source_tag}{ttl_tag}{timeout_tag}</IPV{version}></TraceRoute></Action>\
+        </Destination>{vrf_tag}{source_tag}{ttl_tag}{timeout_tag}</IPV{version}></TraceRoute></Action>\
         </Set>'.format(
             version=ipv,
             destination=destination,
+            vrf_tag=vrf_tag,
             source_tag=source_tag,
             ttl_tag=ttl_tag,
             timeout_tag=timeout_tag
